// app.js
// ------
function App() {
	
	/**
	 * Converts the Date object d into a timestamp that can
	 * be used in SQLite.
	 *
	 * If d is null then the current Date / time is used
	 */
	var convertToSQLiteTime = function(d) {
		var date = d || new Date();
		var time = date.getTime();
		return time / 1000;
	};
	
	return {
		saveTask : function(callback) {
			var task = {};
			task.desc = $('#task-desc').val();
			task.priority = $('#priority').val();
			var dueDate = $('#date-due').val();
			if (dueDate && dueDate.length > 0) {
				task.due_date = dueDate;
			}
			
			task.date_modified = convertToSQLiteTime();
			
			if (task.id && task.id > 0) {
				todo.updateTask(task, function(tx, results) {
          console.log(tx);
          console.log(results);
          callback();
        });
			} else {
				task.date_created = task.date_modified;
				todo.insertTask(task, function(tx, results) {
          console.log(tx);
          console.log(results);
          callback();  
        });
			}
		},
		
		getTask : function(id) {
			todo.getTaskById(id, function(tx, results) {
				if (results.rows.length == 1) {
					var item = results.rows.item(0);
					console.log(item);
				}
			});
		}
	};
}

$(document).ready(function() {
	var app = App(),
      $main = $('#main'),
      $toolbar = $('#toolbar');

  var templates = {
    form : $('#form').html(),
    noTasks : $('#no-tasks').html(),
    toolbarItems : $('#toolbarItems').html(),
    list : $('#list').html(),
    noSupport : $('#not-supported').html()  
  };

  function renderToolbar(data) {
    console.log(data);
    if (data) {
      $toolbar.html(Mustache.render(templates.toolbarItems, data));
    } else {
      $toolbar.hide();
    }
  }
	
  function renderForm(data) {
    renderToolbar({
      links: [
        { href: '#back', text: 'Back' }
      ]
    });
    var output = Mustache.render(templates.form, data);
    $main.html(output);
  }

  function renderList() {
    renderToolbar({
      links: [
        { href: '#edit', text: 'Edit', className: 'edit' },
        { href: '#add', text: 'Add', className: 'add' }
      ]
    });

    todo.getTasks(function(tx, results) {
      var items = [];
      for (var i = 0, total = results.rows.length; i < total; i++) {
        items.push(results.rows.item(i));
      }

      console.log(items);
     
      var tmpl = (items && items.length) ? templates.list : templates.noTasks;
      var output = Mustache.render(tmpl, {
        tasks: items
      });
      $main.html(output);
    });
  }

	var isSupported = todo.initDatabase();
  if (!isSupported) {
    $main.html(Mustache.render(templates.noSupport, {}));
  }

	if (isSupported) {
    $('a[href="#back"]').live('click tap', function(event) {
      event.preventDefault();
      event.stopPropagation();
      renderList();
    });

    $('a.add').live('click tap', function(event) {
      event.stopPropagation();
      event.preventDefault();

      renderForm({
        mode: 'Add',
        task: {
          desc: '',
          priority: 3,
          dateDue: ''
        }
      });
    });

    initSubmitHandler();

		todo.createDatabase(function() {
		  renderList();	
		});
		
		// Init the slider's <output> element
    function initSliderOutput() {
      var priorityOutput = $('#task-priority').get(0);
      priorityOutput.onforminput = function() {
        this.value = $('#priority').val();
      }
      $('#priority').change(function() {
        priorityOutput.innerHTML = this.value;
      });
    }
	
    function initSubmitHandler() {  
      $('#edit-task-form').live('submit', function(event) {
        event.stopPropagation();
        event.preventDefault();
        
        app.saveTask(renderList);
      });
    }
	}
});
