// todo.js
// -------
var todo = function() {
	var db;

  function isFunction(f) {
    // see http://jsperf.com/alternative-isfunction-implementations/4
    return typeof(f) == 'function';
  }
	
	return {
		
		initDatabase : function() {
			if (window.openDatabase) {
				db = openDatabase('todo', '1.1', 'TODO list', 2 * 1024 * 1024);
				return true;
			} else {
				return false;
			}
		},
		
		createDatabase : function(callback) {
			var sql = "CREATE TABLE IF NOT EXISTS Tasks (id INTEGER PRIMARY KEY NOT NULL, desc TEXT NOT NULL, priority INTEGER NOT NULL DEFAULT 3, date_created INTEGER NOT NULL, date_modified INTEGER NOT NULL, date_due INTEGER, date_completed INTEGER)";
			db.transaction(function(tx) {
				tx.executeSql(sql, [], function() {
					callback();
				});
			});
		},
		
		insertTask : function(task, callback) {
			var sql = "INSERT INTO Tasks (desc, priority, date_created, date_modified, date_due) VALUES (?, ?, ?, ?, ?)";
			db.transaction(function(tx) {
				tx.executeSql(sql, [task.desc, task.priority, task.date_created, task.date_modified, task.date_due], function(tx, results) {
					task.id = results.insertId;
          if (isFunction(callback)) {
            callback(tx, results);
          }
				});
			});
		},
		
		updateTask : function(task, callback) {
			var sql = "UPDATE Tasks SET desc = ?, priority = ?, date_modified = ?, date_due = ?, date_completed = ? WHERE id = ?";
			task.date_modified = app.convertToSQLiteTime();
			db.transaction(function(fx) {
				tx.executeSql(sql, [task.desc, task.priority, task.date_modified, task.date_due, task.date_completed, task.id], function(tx, results) {
          if (isFunction(callback)) {
            callback(tx, results);
          }
        });
			});
		},
		
		deleteTask : function(id, callback) {
			var sql = "DELETE FROM Tasks WHERE id = ?";
			db.transaction(function(tx) {
				tx.executeSql(sql, [id], function(tx, results) {
          if (isFunction(callback)) {
            callback(tx, results);
          } 
        });
			});
		},
		
		getTasks : function(callback) {
			var sql = "SELECT * FROM Tasks ORDER BY date_modified DESC";
			db.transaction(function(tx) {
				tx.executeSql(sql, [], function(tx, results){
					callback(tx, results);
				});
			});
		},
		
		getTaskById : function(id, callback) {
			var sql = "SELECT * FROM Tasks WHERE id = ?";
			db.transaction(function(fx) {
				tx.executeSql(sql, [id], function(tx, results) {
					callback(tx, results);
				});
			});
		}
	}
}();
